//
//  LCD Display (SC2004C 8Bit)
//
//  2010.10.16
//
//  PB0-7		LCD DB0-7
//  PD5         LCD E
//  PD6         LCD RS
//  PD7         LED
//
//		_deay_us();  max 96uSec at 8MHz
//		_deay_ms();  max 32mSec at 8MHz
//

#include "cpu_def.h"

#include <avr/io.h>
#include <util/delay.h>

#include "lcd.h"

#define E			5
#define RS			6
#define	LED			7

#define	LCD_X_MAX	19
#define	LCD_Y_MAX	3


static uint8_t lcd_x, lcd_y;
static uint8_t dmode;

static void Lcd_E( void )
{
	PORTD |= _BV(E);
	_delay_us( 50 );							// 50uSec (min 1uSec)
	PORTD &= ~_BV(E);
}

//
//  Out of command
static void Lcd_putcmd( uint8_t cmd )
{
	PORTD &= ~_BV(RS);
	PORTD &= ~_BV(E);

	PORTB = cmd;
	_delay_us( 5 );								// 5uSec (min 195nSec)

	Lcd_E();

}

//
//  Out of data
static void Lcd_putdata( uint8_t data )
{

	PORTD |= _BV(RS);
	PORTD &= ~_BV(E);

	PORTB = data;
	_delay_us( 5 );							// 5uSec (min 195nSec)
	Lcd_E();

}

void LcdTextClear( void )
{
	Lcd_putcmd( 0x01 );						// tmSec (time of Processing 1.52mSec)
	lcd_x = lcd_y = 0;
	_delay_ms( 5 );
}

void LcdTextMode( uint8_t dsp )
{
	switch( dsp ) {
	case LCD_DISPLAY_OFF:					// Display off
		dmode &= ~0b0100;
		Lcd_putcmd( 0x08 | dmode );
		break;
	case LCD_DISPLAY_ON:					// Display on
		dmode |= 0b0100;
		Lcd_putcmd( 0x08 | dmode );
		break;
	case LCD_CURSOR_OFF:					// Cursor off
		dmode &= ~0b0010;  
		Lcd_putcmd( 0x08 | dmode );
		break;
	case LCD_CURSOR_ON:						// Cursor on
		dmode |= 0b0010;
		Lcd_putcmd( 0x08 | dmode );
		break;
	case LCD_BRINK_OFF:						// Cursor brink off
		dmode &= ~0b0001;
		Lcd_putcmd( 0x08 | dmode );
		break;
	case LCD_BRINK_ON:
		dmode |= 0b0001;					// Cursor brink on
		Lcd_putcmd( 0x08 | dmode );
		break;
	}
}

void LcdTextCursor( uint8_t mode )
{
	switch( mode ) {
	case LCD_CURSOR_LEFT:					// Left
		Lcd_putcmd( 0x10 | 0b0000 );
		break;
	case LCD_CURSOR_RIGHT:					// Right
		Lcd_putcmd( 0x10 | 0b0100 );
		break;
	case LCD_CURSOR_LEFT_D:					// Left + dsp
		Lcd_putcmd( 0x10 | 0b1000 );
		break;
	case LCD_CURSOR_RIGHT_D:					// Right + dsp
		Lcd_putcmd( 0x10 | 0b1100 );
		break;
	}
}

static void Lcd_locates( void )
{
	switch( lcd_y ) {
	case 0:
		Lcd_putcmd( 0x80 + 0x00 + lcd_x );
		break;
	case 1:
		Lcd_putcmd( 0x80 + 0x40 + lcd_x );
		break;
	case 2:
		Lcd_putcmd( 0x80 + 0x14 + lcd_x );
		break;
	case 3:
		Lcd_putcmd( 0x80 + 0x54 + lcd_x );
		break;
	}
}

void LcdTextLocate( uint8_t x, uint8_t y )
{
	if (lcd_x > LCD_X_MAX) x = LCD_X_MAX;
	if (lcd_y > LCD_Y_MAX) y = LCD_Y_MAX;
	lcd_x = x;
	lcd_y = y;
	Lcd_locates();
}

void LcdTextPutch( char data )
{
	Lcd_putdata( (uint8_t)data );
	lcd_x++;
	if (lcd_x > LCD_X_MAX) {
		lcd_x = 0;
		lcd_y++;
		if (lcd_y > LCD_Y_MAX) lcd_y = 0;
		Lcd_locates();
	}
}

void LcdTextPutstr( char *data )
{
	while( *data ) LcdTextPutch( *data++ );
}

void LcdTextInitialize( void )
{
	_delay_ms( 30 );				// Wait 30mSec
	Lcd_putcmd( 0x30 );				// Fnction Set 8bit mode
	_delay_ms( 10 );				// Wait 10mSec (min 4.2mSec)
	Lcd_putcmd( 0x30 );
	_delay_ms( 1 );					// Wait  1mSec (min 0.3mSec)
	Lcd_putcmd( 0x30 );

	Lcd_putcmd( 0x3c );				// Fnction Set 8bit,2Line, 5x10dot
	LcdTextMode( LCD_DISPLAY_OFF );	// Display off
	LcdTextClear( );				// Display clear
	_delay_ms( 3 );					// wait 3mSec (min 1.5mSec)

	Lcd_putcmd( 0x06 );				// Entry mode address+1
	LcdTextMode( LCD_DISPLAY_ON );	// Display on
} 

void LcdTextLED( uint8_t dsp )
{
	switch( dsp ) {
	case LCD_LED_OFF:
		PORTD &= ~_BV( LED );
		break;
	case LCD_LED_ON:
		PORTD |= _BV( LED );
		break;
	}
}



